from tables import *
class AnalyticalPhantomMultiScaleReconstruction(IsDescription):
    phantom = Float32Col(shape=(256, 256))
    direct1x = Float32Col(shape=(256, 256))
    direct2x = Float32Col(shape=(256, 256))
    direct4x = Float32Col(shape=(256, 256))
    direct8x = Float32Col(shape=(256, 256))
    infer2x = Float32Col(shape=(256, 256))
    infer4x = Float32Col(shape=(256, 256))
    infer8x = Float32Col(shape=(256, 256))
    iterp2x = Float32Col(shape=(256, 256))
    iterp4x = Float32Col(shape=(256, 256))
    iterp8x = Float32Col(shape=(256, 256))
    direct1x = Float32Col(shape=(256, 256))
    direct2x = Float32Col(shape=(256, 256))
    direct4x = Float32Col(shape=(256, 256))
    direct8x = Float32Col(shape=(256, 256))
    infer2x = Float32Col(shape=(256, 256))
    infer4x = Float32Col(shape=(256, 256))
    infer8x = Float32Col(shape=(256, 256))
    iterp2x = Float32Col(shape=(256, 256))
    iterp4x = Float32Col(shape=(256, 256))
    iterp8x = Float32Col(shape=(256, 256))